﻿/**
* Format class. Provides some general purpose utilities for describing complex objects.
* All methods are static. Do not instantiate.
* @author k0t0vich
*/
class  flash.utils.Format 
{
	private static var CLASS_NAME:String = "className";
	
	/**
	 * Returns a string representation of the object specified in "o" argument.
	 * @param	o	Object. An object to be described.
	 * @return	String. A Flash-debugger like string eg.:
	 * [%object name% %object property%="%property value%"]
	 */
	public static function formatToString(o:Object):String 
	{
		var i:Number = 1;
		var l:Number = arguments.length;
		var r_str:String = "[" + o[CLASS_NAME] + " ";
		for (i = 1; i < l; i++)
		{
			r_str += arguments[i] + "=" + o[arguments[i]] + " ";
		}
		return r_str + "]";
	}
	
	/**
	 * Returns a string representation of the object specified in "o" argument.
	 * @param	o	Object. An object to be described.
	 * @return	String. A CSS like string eg.:
	 * %object name% {%object property%:%property value%;}
	 */
	public static function formatToStyle(o:Object):String 
	{
		var skip:Array = arguments.slice(1, arguments.length);
		var i:Number = skip.length;
		var needSkip:Boolean = false;
		var s:String;
		var r_str:String = o[CLASS_NAME] + " {";
		for (s in o)
		{
			if (o[s] instanceof Function) continue;
			if (s == CLASS_NAME) continue;
				while (i--) 
				{
					if (s == skip[i])
					{
						needSkip = true;
						break;
					}
				}
			i = skip.length;
			if (needSkip) continue;
			r_str += s + ":" + o[s] + "; ";
		}
		return r_str + "}";
	}
	
	/**
	 * JavaScript-compatibility fixer for escape() function.
	 * Returns the string as if it was encoded by encodeURI()
	 * JavaScript function.
	 * @param	encoded	String. The string to be encoded.
	 * @return	String.
	 */
	public static function encodeURI(encoded:String):String
	{
		return escape(encoded).split("%2A").join("*").split("%40").join("@").split("%2D").join(
		"-").split("%5F").join("_").split("%2B").join("+").split("%2E").join(".").split("%2F").join("/");
	}
	
	/**
	 * Restores the string encoded by encodeURI function.
	 * @param	decoded	String. The string to be restored from previously decoded by encodeURI.
	 * @return
	 */
	public static function decodeURI(decoded:String):String
	{
		return(unescape(decoded));
	}

}